#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.cbr.v3 import policy
from otcextensions.sdk.cbr.v3 import policy as policySDK
from otcextensions.tests.unit.osclient.cbr.v3 import fakes


class TestPolicy(fakes.TestCBR):

    def setUp(self):
        super(TestPolicy, self).setUp()

    def test_flatten(self):
        obj = fakes.FakePolicy.create_one()

        flat_data = policy._flatten_policy(obj)

        data = (
            flat_data['id'],
            flat_data['name'],
            flat_data['enabled'],
            flat_data['operation_type'],
            flat_data['retention_duration_days'],
            flat_data['max_backups'],
            flat_data['year_backups'],
            flat_data['day_backups'],
            flat_data['week_backups'],
            flat_data['month_backups'],
            flat_data['timezone'],
            flat_data['start_time']
        )

        od = obj.operation_definition
        cmp_data = (
            obj.id,
            obj.name,
            obj.enabled,
            obj.operation_type,
            od.retention_duration_days,
            od.max_backups,
            od.year_backups,
            od.day_backups,
            od.week_backups,
            od.month_backups,
            od.timezone,
            obj.trigger.properties.start_time,
        )

        self.assertEqual(data, cmp_data)

    def test_add_vaults_to_policy_output(self):
        obj = fakes.FakePolicyFixed.create_one()

        column = ()
        data = ()
        verify_column = (
            'associated_vault_1',
            'associated_vault_2',
            'associated_vault_3',
        )
        verify_data = (
            'vault_id_1',
            'vault_id_2',
            'vault_id_3',
        )

        data, column = policy._add_vaults_to_policy_obj(obj, data, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)

    def test_add_scheduling_patterns(self):
        obj = fakes.FakePolicyFixed.create_one()

        column = ()
        data = ()
        verify_column = (
            'schedule_pattern_1',
            'schedule_pattern_2'
        )
        verify_data = (
            'pattern_1',
            'pattern_2'
        )

        data, column = policy._add_scheduling_patterns(obj, data, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)


class TestListPolicy(fakes.TestCBR):

    objects = fakes.FakePolicy.create_multiple(3)

    columns = ('ID', 'name', 'operation_type', 'start_time', 'enabled')

    data = []

    for s in objects:
        flat_data = policy._flatten_policy(s)
        data.append((
            flat_data['id'],
            flat_data['name'],
            flat_data['operation_type'],
            flat_data['start_time'],
            flat_data['enabled']
        ))

    def setUp(self):
        super(TestListPolicy, self).setUp()

        self.cmd = policy.ListPolicies(self.app, None)

        self.client.policies = mock.Mock()
        self.client.api_mock = self.client.policies

    def test_default(self):
        arglist = [
            '--vault-id', 'vault_id',
            '--operation-type', 'backup'
        ]

        verifylist = [
            ('vault_id', 'vault_id'),
            ('operation_type', 'backup')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            vault_id='vault_id',
            operation_type='backup',
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowPolicy(fakes.TestCBR):

    object = fakes.FakePolicy.create_one()

    columns = (
        'ID',
        'name',
        'operation_type',
        'start_time',
        'enabled',
        'retention_duration_days',
        'max_backups',
        'day_backups',
        'week_backups',
        'month_backups',
        'year_backups',
        'timezone',
    )

    flat_data = policy._flatten_policy(object)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['operation_type'],
        flat_data['start_time'],
        flat_data['enabled'],
        flat_data['retention_duration_days'],
        flat_data['max_backups'],
        flat_data['day_backups'],
        flat_data['week_backups'],
        flat_data['month_backups'],
        flat_data['year_backups'],
        flat_data['timezone'],
    )

    def setUp(self):
        super(TestShowPolicy, self).setUp()

        self.cmd = policy.ShowPolicy(self.app, None)

        self.client.find_policy = mock.Mock()

    def test_default(self):
        arglist = [
            'policy'
        ]
        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_policy.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_policy.assert_called_once_with(
            name_or_id='policy',
            ignore_missing=False,
        )

        self.data, self.columns = policy._add_vaults_to_policy_obj(
            self.object,
            self.data,
            self.columns
        )

        self.data, self.columns = policy._add_scheduling_patterns(
            self.object,
            self.data,
            self.columns
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeletePolicy(fakes.TestCBR):

    def setUp(self):
        super(TestDeletePolicy, self).setUp()

        self.cmd = policy.DeletePolicy(self.app, None)

        self.client.delete_policy = mock.Mock()

    def test_delete(self):
        arglist = [
            'p1'
        ]
        verifylist = [
            ('policy', 'p1')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_policy.side_effect = [{}]

        # Set the response for find_policy
        self.client.find_policy.side_effect = [
            policySDK.Policy(id='p1')
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        delete_calls = [
            mock.call(
                policy='p1',
                ignore_missing=False),
        ]

        find_calls = [
            mock.call(
                name_or_id='p1',
                ignore_missing=False),
        ]

        self.client.delete_policy.assert_has_calls(delete_calls)
        self.client.find_policy.assert_has_calls(find_calls)
        self.assertEqual(1, self.client.delete_policy.call_count)


class TestCreatePolicy(fakes.TestCBR):

    object = fakes.FakePolicy.create_one()

    columns = (
        'ID',
        'name',
        'operation_type',
        'start_time',
        'enabled',
        'retention_duration_days',
        'max_backups',
        'day_backups',
        'week_backups',
        'month_backups',
        'year_backups',
        'timezone',
    )

    flat_data = policy._flatten_policy(object)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['operation_type'],
        flat_data['start_time'],
        flat_data['enabled'],
        flat_data['retention_duration_days'],
        flat_data['max_backups'],
        flat_data['day_backups'],
        flat_data['week_backups'],
        flat_data['month_backups'],
        flat_data['year_backups'],
        flat_data['timezone'],
    )

    def setUp(self):
        super(TestCreatePolicy, self).setUp()

        self.cmd = policy.CreatePolicy(self.app, None)
        self.app.client_manager.sdk_connection = mock.Mock()

        self.client.create_policy = mock.Mock()

    def test_default(self):
        arglist = [
            'policy_name',
            '--disable',
            '--operation-type', 'backup',
            '--pattern', 'pattern_1',
            '--pattern', 'pattern_2',
            '--day-backups', '1',
            '--week-backups', '2',
            '--month-backups', '3',
            '--year-backups', '4',
            '--timezone', 'tz',
            '--max-backups', '10',
            '--retention-duration-days', '9'
        ]
        verifylist = [
            ('name', 'policy_name'),
            ('disable', False),
            ('operation_type', 'backup'),
            ('patterns', ['pattern_1', 'pattern_2']),
            ('day_backups', 1),
            ('week_backups', 2),
            ('month_backups', 3),
            ('year_backups', 4),
            ('timezone', 'tz'),
            ('max_backups', 10),
            ('retention_duration_days', 9),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_policy.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_policy.assert_called_once_with(
            operation_definition={
                'day_backups': 1,
                'week_backups': 2,
                'month_backups': 3,
                'year_backups': 4,
                'max_backups': 10,
                'retention_duration_days': 9,
                'timezone': 'tz'},
            trigger={
                'properties': {
                    'pattern': ['pattern_1', 'pattern_2']}},
            name='policy_name',
            enabled=False,
            operation_type='backup'
        )

        self.data, self.columns = policy._add_scheduling_patterns(
            self.object,
            self.data,
            self.columns
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestUpdatePolicy(fakes.TestCBR):

    object = fakes.FakePolicy.create_one()

    columns = (
        'ID',
        'name',
        'operation_type',
        'start_time',
        'enabled',
        'retention_duration_days',
        'max_backups',
        'day_backups',
        'week_backups',
        'month_backups',
        'year_backups',
        'timezone',
    )

    flat_data = policy._flatten_policy(object)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['operation_type'],
        flat_data['start_time'],
        flat_data['enabled'],
        flat_data['retention_duration_days'],
        flat_data['max_backups'],
        flat_data['day_backups'],
        flat_data['week_backups'],
        flat_data['month_backups'],
        flat_data['year_backups'],
        flat_data['timezone'],
    )

    def setUp(self):
        super(TestUpdatePolicy, self).setUp()

        self.cmd = policy.UpdatePolicy(self.app, None)
        self.app.client_manager.sdk_connection = mock.Mock()

        self.client.update_policy = mock.Mock()

    def test_default(self):
        arglist = [
            'policy_id',
            '--name', 'pol1',
            '--enable',
            '--pattern', 'pattern_1',
            '--pattern', 'pattern_2',
            '--day-backups', '1',
            '--week-backups', '2',
            '--month-backups', '3',
            '--year-backups', '4',
            '--timezone', 'tz',
            '--max-backups', '10',
            '--retention-duration-days', '9'
        ]
        verifylist = [
            ('policy', 'policy_id'),
            ('name', 'pol1'),
            ('enable', True),
            ('patterns', ['pattern_1', 'pattern_2']),
            ('day_backups', 1),
            ('week_backups', 2),
            ('month_backups', 3),
            ('year_backups', 4),
            ('timezone', 'tz'),
            ('max_backups', 10),
            ('retention_duration_days', 9),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_policy.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_policy.assert_called_with(
            name_or_id='policy_id',
            ignore_missing=False)

        self.client.update_policy.assert_called_once_with(
            policy=mock.ANY,
            name='pol1',
            enabled=True,
            trigger={
                'properties': {
                    'pattern': ['pattern_1', 'pattern_2']}},
            operation_definition={
                'day_backups': 1,
                'week_backups': 2,
                'month_backups': 3,
                'year_backups': 4,
                'max_backups': 10,
                'retention_duration_days': 9,
                'timezone': 'tz'}
        )

        self.data, self.columns = policy._add_vaults_to_policy_obj(
            self.object,
            self.data,
            self.columns
        )

        self.data, self.columns = policy._add_scheduling_patterns(
            self.object,
            self.data,
            self.columns
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
# import datetime
import random
import uuid
import time

import mock

from openstackclient.tests.unit import utils
from otcextensions.tests.unit.osclient import test_base

from otcextensions.sdk.cts.v1 import trace
from otcextensions.sdk.cts.v1 import tracker


class TestCTS(utils.TestCommand):

    def setUp(self):
        super(TestCTS, self).setUp()

        self.app.client_manager.cts = mock.Mock()
        self.client = self.app.client_manager.cts

        self.client.traces = mock.Mock()


class FakeTrace(test_base.Fake):
    """Fake one or more Trace"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': uuid.uuid4().hex,
            'level': uuid.uuid4().hex,
            'type': uuid.uuid4().hex,
            'time': time.time() * 1000,
            'user': uuid.uuid4().hex,
            'code': random.randint(1, 600),
            'service_type': uuid.uuid4().hex,
            'resource_type': uuid.uuid4().hex,
            'resource_name': uuid.uuid4().hex,
            'resource_id': uuid.uuid4().hex,
            'source_ip': uuid.uuid4().hex,
            'record_time': time.time() * 1000,
            'request': uuid.uuid4().hex,
            'response': uuid.uuid4().hex,
        }
        obj = trace.Trace.existing(**object_info)
        return obj


class FakeTracker(test_base.Fake):
    """Fake one or more Tracker"""

    @classmethod
    def generate(cls):
        object_info = {
            'name': uuid.uuid4().hex,
            'bucket_name': uuid.uuid4().hex,
            'file_prefix_name': uuid.uuid4().hex,
            'status': uuid.uuid4().hex,
            'details': uuid.uuid4().hex,
            'smn': {
                'enabled': True,
                'topic_id': uuid.uuid4().hex,
                'is_send_all_key_operation': True,
                'need_notify_user_list': [uuid.uuid4().hex, uuid.uuid4().hex],
                'operations': [uuid.uuid4().hex, uuid.uuid4().hex]
            }
        }
        obj = tracker.Tracker.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.rds.v3 import datastore

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {"id": IDENTIFIER, "name": "5.7"}


class TestDatastore(base.TestCase):
    def setUp(self):
        super(TestDatastore, self).setUp()

    def test_basic(self):
        sot = datastore.Datastore()
        self.assertEqual('dataStores', sot.resources_key)
        self.assertEqual('/datastores/%(database_name)s', sot.base_path)
        self.assertIsNone(sot.resource_key)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)
        self.assertDictEqual({
            'limit': 'limit',
            'marker': 'marker'
        }, sot._query_mapping._mapping)

    def test_make_it(self):
        sot = datastore.Datastore(**EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.imsv1.v1 import async_job

EXAMPLE = {
    "status": "SUCCESS",
    "job_id": "ff8080814dbd65d7014dbe0d84db0013",
    "job_type": "createImageByInstance",
    "begin_time": "04-Jun-2015 18:11:06:586",
    "end_time": "",
    "error_code": "404",
    "fail_reason": "something bad happened"
}


class TestAsyncJob(base.TestCase):

    def test_basic(self):
        sot = async_job.AsyncJob()
        path = ''
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):
        sot = async_job.AsyncJob(**EXAMPLE)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['job_id'], sot.job_id)
        self.assertEqual(EXAMPLE['job_type'], sot.job_type)
        self.assertEqual(EXAMPLE['begin_time'], sot.begin_time)
        self.assertEqual(EXAMPLE['end_time'], sot.end_time)
        self.assertEqual(EXAMPLE['error_code'], sot.error_code)
        self.assertEqual(EXAMPLE['fail_reason'], sot.fail_reason)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from openstack import resource
from openstack import _log

from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestServerTag(base.BaseFunctionalTest):
    cidr = '192.168.0.0/16'
    uuid_v4 = uuid.uuid4().hex[:8]
    vpc_name = 'server-test-vpc-' + uuid_v4
    subnet_name = 'server-test-subnet-' + uuid_v4
    server_name = 'server-test-' + uuid_v4
    kp_name = 'server-test-kp-' + uuid_v4
    image_name = 'Standard_Fedora_34_latest'
    flavor_name = 's3.medium.1'

    def setUp(self):
        super(TestServerTag, self).setUp()
        self.vpc = self.conn.vpc.create_vpc(
            name=self.vpc_name,
            cidr=self.cidr
        )
        self.assertEqual(self.vpc_name, self.vpc.name)
        gw, _ = self.cidr.split("/")
        self.subnet = self.conn.vpc.create_subnet(
            name=self.subnet_name,
            vpc_id=self.vpc.id,
            cidr=self.vpc.cidr,
            gateway_ip=gw[:-2] + ".1",
            dns_list=[
                "100.125.4.25",
                "100.125.129.199",
            ]
        )
        resource.wait_for_status(
            self.conn.vpc, self.subnet, "ACTIVE", None, 2, 20
        )
        self.assertEqual(self.subnet_name, self.subnet.name)

        image = self.conn.compute.find_image(self.image_name)
        flavor = self.conn.compute.find_flavor(self.flavor_name)
        self.keypair = self.conn.compute.create_keypair(name=self.kp_name)
        self.server = self.conn.compute.create_server(
            name=self.server_name,
            image_id=image.id,
            flavor_id=flavor.id,
            networks=[{"uuid": self.subnet.neutron_network_id}],
            key_name=self.keypair.name
        )
        self.server = self.conn.compute.wait_for_server(self.server)

    def tearDown(self):
        kp = self.conn.compute.find_keypair(self.kp_name)
        srv = self.conn.compute.find_server(self.server_name)
        self.conn.compute.delete_server(srv)
        self.conn.compute.wait_for_delete(
            srv, interval=5, wait=600)
        self.conn.compute.delete_keypair(kp)
        self.conn.compute.wait_for_delete(kp)

        resource.wait_for_status(
            self.conn.vpc, self.subnet, "ACTIVE", None, 2, 20
        )
        self.conn.vpc.delete_subnet(self.subnet, ignore_missing=False)
        resource.wait_for_delete(self.conn.vpc, self.subnet, 2, 60)

        sot = self.conn.vpc.delete_vpc(self.vpc)
        self.assertIsNotNone(sot)
        super(TestServerTag, self).tearDown()

    def test_tags_workflow(self):
        instance = self.conn.compute.get_server(self.server)
        self.assertEqual(len(instance.tags), 0)
        self.server.add_tag(self.conn.compute, 'pytest=test1')
        self.assertEqual(len(instance.tags), 1)
        self.server.remove_tag(self.conn.compute, 'pytest=test1')
        self.assertEqual(len(instance.tags), 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import apienvironmentvar as var

EXAMPLE_VAR = {
    "variable_value": "192.168.1.5",
    "env_id": "7a1ad0c350844ee69479b47df9a881cb",
    "group_id": "c77f5e81d9cb4424bf704ef2b0ac7600",
    "id": "25054838a624400bbf2267cf5b3a3f70",
    "variable_name": "address"
}


class TestApiEnvironment(base.TestCase):

    def test_basic(self):
        sot = var.ApiEnvironmentVar()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/env-variables',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_fetch)
        self.assertEqual('variables', sot.resources_key)

    def test_make_it(self):
        sot = var.ApiEnvironmentVar(**EXAMPLE_VAR)
        self.assertEqual(EXAMPLE_VAR['variable_value'], sot.variable_value)
        self.assertEqual(EXAMPLE_VAR['env_id'], sot.env_id)
        self.assertEqual(EXAMPLE_VAR['group_id'], sot.group_id)
        self.assertEqual(EXAMPLE_VAR['id'], sot.id)
        self.assertEqual(EXAMPLE_VAR['variable_name'], sot.variable_name)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import json
import uuid

from datetime import datetime

from openstackclient.tests.functional import base


class NatTestCase(base.TestCase):
    """Common functional test bits for NAT commands"""

    CURR_TIME = datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")

    def setUp(self):
        super(NatTestCase, self).setUp()
        UUID = uuid.uuid4().hex[:8]
        self.ROUTER_NAME = 'otce-nat-test-router-' + UUID
        self.NETWORK_NAME = 'otce-nat-test-net-' + UUID
        self.SUBNET_NAME = 'otce-nat-test-subnet-' + UUID
        self.NAT_NAME = 'otce-nat-test-' + UUID

        self.ROUTER_ID = None
        self.NETWORK_ID = None
        self.FLOATING_IP_ID = None
        self.NAT_ID = None
        self.SNAT_RULE_ID = None
        self.DNAT_RULE_ID = None

    def create_nat_gateway(self, name=None):
        self._initialize_network()
        name = name or self.NAT_NAME
        json_output = json.loads(self.openstack(
            'nat gateway create {name}'
            ' --router-id {router_id}'
            ' --internal-network-id {net_id}'
            ' --description {description}'
            ' --spec {spec} -f json'.format(
                name=name,
                router_id=self.ROUTER_ID,
                net_id=self.NETWORK_ID,
                description='OTCE_func_test',
                spec=1)
        ))
        self.assertIsNotNone(json_output)
        self.NAT_ID = json_output['id']
        return json_output

    def delete_nat_gateway(self):
        self.addCleanup(self._denitialize_network)
        self.openstack('nat gateway delete ' + self.NAT_ID)

    def create_snat_rule(self):
        nat_gateway = self.create_nat_gateway()
        self.assertIsNotNone(nat_gateway)
        self.assertIsNotNone(self.FLOATING_IP_ID)
        json_output = json.loads(self.openstack(
            'nat snat rule create '
            '--nat-gateway-id {nat_gateway_id} '
            '--floating-ip-id {floating_ip_id} '
            '--network-id {net_id} -f json'.format(
                nat_gateway_id=nat_gateway['id'],
                floating_ip_id=self.FLOATING_IP_ID,
                net_id=self.NETWORK_ID)
        ))
        self.assertIsNotNone(json_output)
        self.SNAT_RULE_ID = json_output['id']
        return json_output

    def delete_snat_rule(self):
        self.addCleanup(self.delete_nat_gateway)
        self.openstack(
            'nat snat rule delete ' + self.SNAT_RULE_ID)

    def create_dnat_rule(self):
        nat_gateway = self.create_nat_gateway()
        self.assertIsNotNone(nat_gateway)
        self.assertIsNotNone(self.FLOATING_IP_ID)
        json_output = json.loads(self.openstack(
            'nat dnat rule create '
            '--nat-gateway-id {nat_gateway_id} '
            '--floating-ip-id {floating_ip_id} '
            '--protocol {protocol} '
            '--internal-service-port 80 '
            '--external-service-port 80 '
            '--private-ip {private_ip} '
            '-f json'.format(
                nat_gateway_id=nat_gateway['id'],
                protocol='TCP',
                private_ip='192.168.0.3',
                floating_ip_id=self.FLOATING_IP_ID)
        ))
        self.assertIsNotNone(json_output)
        self.DNAT_RULE_ID = json_output['id']
        return json_output

    def delete_dnat_rule(self):
        self.addCleanup(self.delete_nat_gateway)
        self.openstack(
            'nat dnat rule delete ' + self.DNAT_RULE_ID)

    def _initialize_network(self):
        router = json.loads(self.openstack(
            'router create -f json ' + self.ROUTER_NAME
        ))
        net = json.loads(self.openstack(
            'network create -f json ' + self.NETWORK_NAME
        ))
        self.openstack(
            'subnet create {subnet} -f json '
            '--network {net} '
            '--subnet-range 192.168.0.0/24 '.format(
                subnet=self.SUBNET_NAME,
                net=self.NETWORK_NAME
            ))

        self.openstack(
            'router add subnet {router} '
            '{subnet} '.format(
                router=self.ROUTER_NAME,
                subnet=self.SUBNET_NAME
            )
        )

        floating_ip = json.loads(self.openstack(
            'floating ip create -f json '
            '{network}'.format(
                network='admin_external_net')
        ))

        self.ROUTER_ID = router['id']
        self.NETWORK_ID = net['id']
        self.FLOATING_IP_ID = floating_ip['id']

    def _denitialize_network(self):
        self.openstack(
            'floating ip delete ' + self.FLOATING_IP_ID
        )
        self.openstack(
            'router remove subnet {router} '
            '{subnet} '.format(
                router=self.ROUTER_NAME,
                subnet=self.SUBNET_NAME
            )
        )
        self.openstack(
            'subnet delete ' + self.SUBNET_NAME
        )
        self.openstack(
            'network delete ' + self.NETWORK_NAME
        )
        self.openstack(
            'router delete ' + self.ROUTER_NAME
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestACPolicy(TestApiG):
    gateway_id = "560de602c9f74969a05ff01d401a53ed"
    acl_id = None

    def setUp(self):
        super(TestACPolicy, self).setUp()

    def test_create_acl_policy(self):
        attrs = {
            "acl_name": "acl_demo",
            "acl_type": "PERMIT",
            "acl_value": "192.168.1.5,192.168.10.1",
            "entity_type": "IP"
        }
        created = self.client.create_acl_policy(
            gateway=TestACPolicy.gateway_id,
            **attrs)
        self.assertIsNotNone(created.id)
        TestACPolicy.acl_id = created.id

    def test_update_acl_policy(self):
        check = "DENY"
        attrs = {
            "acl_name": "acl_demo",
            "entity_type": "IP",
            "acl_type": check,
            "acl_value": "192.168.1.5,192.168.10.1"
        }
        updated = self.client.update_acl_policy(
            gateway=TestACPolicy.gateway_id,
            acl_policy=TestACPolicy.acl_id,
            **attrs)
        self.assertEqual(updated.acl_type, check)

    def test_list_acl_policies(self):
        found = list(self.client.acl_policies(gateway=TestACPolicy.gateway_id))
        self.assertGreater(len(found), 0)

    def test_get_acl_policy(self):
        found = self.client.get_acl_policy(gateway=TestACPolicy.gateway_id,
                                           acl_policy=TestACPolicy.acl_id)
        self.assertIsNotNone(found.id)

    def test_delete_ac_policy(self):
        self.client.delete_acl_policy(gateway=TestACPolicy.gateway_id,
                                      ac_policy=TestACPolicy.acl_id)
        found = self.client.acl_policies(gateway=TestACPolicy.gateway_id)
        self.assertEqual(len(list(found)), 0)

    def test_delete_acl_policies(self):
        attrs = {
            "acl_name": "acl_demo1",
            "acl_type": "PERMIT",
            "acl_value": "192.168.1.5,192.168.10.1",
            "entity_type": "IP"
        }
        created = self.client.create_acl_policy(
            gateway=TestACPolicy.gateway_id,
            **attrs)
        attrs = {
            'acls': [TestACPolicy.acl_id, created.id]
        }
        self.client.delete_acl_policies(gateway=TestACPolicy.gateway_id,
                                        **attrs)
        found = self.client.acl_policies(gateway=TestACPolicy.gateway_id)
        self.assertEqual(len(list(found)), 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource


class CurrentNodeDetailSpec(resource.Resource):
    #: Start time of the current task.
    begin_time = resource.Body('beginTime')
    #: Description of the current task.
    desc = resource.Body('desc')
    #: End time of the current task.
    end_time = resource.Body('endTime')
    #: Upgrade task name.
    name = resource.Body('name')
    #: Sequence number of an upgrade task.
    order = resource.Body('order', type=int)
    #: Status of the current task.
    status = resource.Body('status')


class TargetImageSpec(resource.Resource):
    #: Image engine type.
    datastore_type = resource.Body('datastoreType')
    #: Image engine version.
    datastore_version = resource.Body('datastoreVersion')
    #: Name of an image that can be upgraded.
    display_name = resource.Body('displayName')
    #: ID of an image that can be upgraded.
    id = resource.Body('id')
    #: Image description.
    image_desc = resource.Body('imageDesc')
    #: Priority.
    priority = resource.Body('priority', type=int)


class ClusterUpgradeStatus(resource.Resource):
    base_path = '/clusters/%(cluster_id)s/upgrade/detail'

    allow_list = True

    resources_key = 'detailList'

    _query_mapping = resource.QueryParameters('start', 'limit', 'action_node')

    cluster_id = resource.URI('cluster_id')

    #: Agency name.
    agency_name = resource.Body('agencyName')
    #: Names of the nodes that have been upgraded.
    completed_nodes = resource.Body('completedNodes')
    #: Task details of the node that is being upgraded.
    current_node_detail = resource.Body(
        'currentNodeDetail', type=list, list_type=CurrentNodeDetailSpec
    )
    #: Names of the nodes that are being upgraded.
    current_node_name = resource.Body('currentNodeName')
    #: End time of the upgrade.
    end_time = resource.Body('endTime')
    #: Retried times.
    execute_times = resource.Body('executeTimes')
    #:
    fail_message = resource.Body('failMessage')
    #: Expected result of the cluster upgrade.
    final_az_info_map = resource.Body('finalAzInfoMap')
    #: Task ID.
    id = resource.Body('id')
    #: Image details.
    image_info = resource.Body('imageInfo', type=TargetImageSpec)
    #: Current upgrade behavior of the cluster.
    migrate_param = resource.Body('migrateParam')
    #: Start time of the upgrade.
    start_time = resource.Body('startTime')
    #: Task status.
    status = resource.Body('status')
    #: Names of the nodes to be upgraded.
    total_nodes = resource.Body('totalNodes')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class L7Policy(resource.Resource):
    resource_key = 'l7policy'
    resources_key = 'l7policies'
    base_path = '/elb/l7policies'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'action', 'admin_state_up', 'description',
        'display_all_rules', 'id', 'listener_id',
        'name', 'position', 'provisioning_status',
        'priority', 'redirect_url', 'redirect_pool_id',
        'redirect_listener_id',
        is_admin_state_up='admin_state_up',
    )

    # Properties
    #: Specifies where requests will be forwarded.
    action = resource.Body('action')
    #: Specifies the time when the forwarding policy was added.
    created_at = resource.Body('created_at')
    #: Provides supplementary information about the forwarding policy.
    description = resource.Body('description')
    #: Specifies the administrative status of the forwarding policy.
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Specifies the ID of the listener to which the forwarding policy is
    #: added.
    listener_id = resource.Body('listener_id')
    #: Specifies the forwarding policy name.
    name = resource.Body('name')
    #: Specifies the forwarding policy priority.
    priority = resource.Body('priority', type=int)
    #: Specifies the ID of the project where the forwarding policy is used.
    project_id = resource.Body('project_id')
    #: Specifies the provisioning status of the forwarding policy.
    provisioning_status = resource.Body('provisioning_status')
    #: Specifies the ID of the backend server group that requests are
    #: forwarded to.
    redirect_pool_id = resource.Body('redirect_pool_id')
    #: Specifies the forwarding policy priority.
    position = resource.Body('position', type=int)
    # #: Specifies the configuration of the backend server group that
    # # the requests are forwarded to.
    # redirect_pools_config = resource.Body('redirect_pools_config', tyr=list,
    #                                       elements=dict)
    #: Specifies the ID of the listener to which requests are redirected.
    redirect_listener_id = resource.Body('redirect_listener_id')
    #: Specifies the URL to which requests are forwarded.
    redirect_url = resource.Body('redirect_url')
    #: Lists the forwarding rules in the forwarding policy.
    rules = resource.Body('rules', type=list)
    #: Specifies the URL to which requests are forwarded.
    redirect_url_config = resource.Body('redirect_url_config', type=dict)
    #: Specifies the configuration of the page that will be returned.
    fixed_response_config = resource.Body('fixed_response_config', type=dict)
    #: Specifies the time when the forwarding policy was updated.
    updated_at = resource.Body('updated_at')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class DatapointsSpec(resource.Resource):

    # Properties
    # Metric Value, the value is the same as that of
    # parameter filter
    average = resource.Body('average')
    maximum = resource.Body('max')
    minimum = resource.Body('min')
    sumspec = resource.Body('sum')
    variance = resource.Body('variance')
    # time of the metric collection
    timestamp = resource.Body('timestamp')
    # indicates the metric unit
    unit = resource.Body('unit')


class MetricData(resource.Resource):

    resource_key = ''
    resources_key = ''
    base_path = '/metric-data'

    # capabilities
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'namespace', 'period', 'filter', 'metric_name', 'dim.0', 'from',
        'to')

    # Properties
    # Datapoints retrieve the metrics data list
    datapoints = resource.Body('datapoints', type=list,
                               list_type=DatapointsSpec)
    # Metric Name like 'cpu_util'
    metric_name = resource.Body('metric_name')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestThrottlingPolicy(TestApiG):
    environment = None

    def setUp(self):
        super(TestThrottlingPolicy, self).setUp()
        self.create_gateway()
        self.gateway_id = TestThrottlingPolicy.gateway.id
        # self.gateway_id = "be76ca6de5fe4aa7af503c03b3b44dea"
        self.attrs = {
            "api_call_limits": 100,
            "app_call_limits": 60,
            "enable_adaptive_control": "FALSE",
            "ip_call_limits": 60,
            "name": "throttle_demo",
            "remark": "Total: 800 calls/second;"
                      " user: 500 calls/second;"
                      " app: 300 calls/second;"
                      " IP address: 600 calls/second",
            "time_interval": 1,
            "time_unit": "SECOND",
            "type": 1,
            "user_call_limits": 60
        }
        self.policy = self.client.create_throttling_policy(
            gateway=self.gateway_id,
            **self.attrs
        )

        self.addCleanup(
            self.client.delete_throttling_policy,
            gateway=self.gateway_id,
            policy=self.policy
        )
        self.addCleanup(self.delete_gateway())

    def test_list_throttling_policies(self):
        pol = list(self.client.throttling_policies(
            gateway=self.gateway_id))
        self.assertEqual(len(pol), 1)

    def test_get_throttling_policy(self):
        pol = self.client.get_throttling_policy(
            gateway=self.gateway_id,
            policy=self.policy.id)
        self.assertEqual(pol.id, self.policy.id)

    def test_update_throttling_policy(self):
        attrs = {
            "time_unit": "SECOND",
            "name": "throttle_demo",
            "api_call_limits": 100,
            "time_interval": 1,
            "remark": "Total: 800 calls/second;"
                      " user: 500 calls/second;"
                      " app: 300 calls/second;"
                      " IP address: 600 calls/second",
        }
        updated = self.client.update_throttling_policy(
            gateway=self.gateway_id,
            policy=self.policy.id,
            **attrs
        )
        self.assertEqual(updated.remark, attrs["remark"])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource


class JobResourceSpecifications(resource.Resource):
    base_path = "job/resource-specs"
    resources_key = "specs"

    allow_list = True
    allow_fetch = True
    allow_patch = True

    #: Job type
    job_type = resource.Body("job_type", type=str)

    #: Engine ID of a job
    engine_id = resource.Body("engine_id", type=int)

    #: Project type
    project_type = resource.Body("project_type", type=int)

    #: Whether the request is successful
    is_success = resource.Body("is_success", type=bool)

    #: Error message of a failed API call
    error_message = resource.Body("error_message", type=str)

    #: Error code of a failed API call
    error_code = resource.Body("error_code", type=str)

    #: Total number of job resource specifications
    spec_total_count = resource.Body("spec_total_count", type=int)

    #: List of resource specifications attributes
    specs = resource.Body("specs", type=list)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from otcextensions.osclient.rds.v1 import backup
from otcextensions.tests.unit.osclient.rds.v1 import fakes as fakes


class TestList(fakes.TestRds):

    _objects = fakes.FakeBackup.create_multiple(3)

    columns = ('ID', 'Name', 'instance_id', 'datastore_type',
               'datastore_version',
               'size', 'status', 'created', 'updated')

    data = []

    for s in _objects:
        data.append((
            s.id,
            s.name,
            s.instance_id,
            s.datastore['type'],
            s.datastore['version'],
            s.size,
            s.status,
            s.created,
            s.updated
        ))

    def setUp(self):
        super(TestList, self).setUp()

        self.cmd = backup.ListBackup(self.app, None)

        self.client.backups = mock.Mock()

    def test_list_default(self):
        arglist = [
        ]

        verifylist = [
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.backups.side_effect = [
            self._objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.backups.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestCreate(fakes.TestRds):

    _obj = fakes.FakeBackup.create_one()

    columns = ('ID', 'Name', 'instance_id', 'datastore_type',
               'datastore_version',
               'size', 'status', 'created', 'updated')

    data = (
        _obj.id,
        _obj.name,
        _obj.instance_id,
        _obj.datastore['type'],
        _obj.datastore['version'],
        _obj.size,
        _obj.status,
        _obj.created,
        _obj.updated,
    )

    def setUp(self):
        super(TestCreate, self).setUp()

        self.cmd = backup.CreateBackup(self.app, None)

        self.client.create_backup = mock.Mock()

    def test_create(self):
        arglist = [
            '--name', 'test_name',
            '--description', 'test description',
            'instance_id'
        ]
        verifylist = [
            ('name', 'test_name'),
            ('description', 'test description'),
            ('instance', 'instance_id'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_backup.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_backup.assert_called_with(
            description='test description',
            instance='instance_id',
            name='test_name'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDelete(fakes.TestRds):

    def setUp(self):
        super(TestDelete, self).setUp()

        self.cmd = backup.DeleteBackup(self.app, None)

        self.client.delete_backup = mock.Mock()

    def test_delete_default(self):
        arglist = [
            'bck1',
        ]

        verifylist = [
            ('backup', ['bck1']),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_backup.side_effect = [
            {}
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.delete_backup.assert_called_once_with(
            backup='bck1',
            ignore_missing=False
        )

    def test_delete_multiple(self):
        arglist = [
            'bck1',
            'bck2'
        ]

        verifylist = [
            ('backup', ['bck1', 'bck2']),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_backup.side_effect = [{}, {}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [
            mock.call(backup='bck1', ignore_missing=False),
            mock.call(backup='bck2', ignore_missing=False),
        ]

        self.client.delete_backup.assert_has_calls(calls)
        self.assertEqual(2, self.client.delete_backup.call_count)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from unittest.mock import call

from osc_lib import exceptions

from otcextensions.osclient.dis.v2 import dump_task
from otcextensions.common import cli_utils
from otcextensions.tests.unit.osclient.dis.v2 import fakes

from openstackclient.tests.unit import utils as tests_utils


class TestListDumpTasks(fakes.TestDis):

    objects = fakes.FakeDumpTask.create_multiple(3)

    column_list_headers = (
        'Task Name',
        'Task Id',
        'Destination Type',
        'Created At',
        'Status'
    )

    columns = (
        'task_name',
        'task_id',
        'destination_type',
        'created_at',
        'status',
    )

    data = []

    for s in objects:
        data.append((
            s.task_name,
            s.task_id,
            s.destination_type,
            cli_utils.UnixTimestampFormatter(s.created_at),
            s.status
        ))

    def setUp(self):
        super(TestListDumpTasks, self).setUp()

        self.cmd = dump_task.ListDumpTasks(self.app, None)

        self.client.dump_tasks = mock.Mock()
        self.client.api_mock = self.client.dump_tasks

    def test_list(self):
        arglist = [
            'test-stream',
        ]

        verifylist = [
            ('streamName', 'test-stream'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with('test-stream')

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))


class TestCreateDumpTask(fakes.TestDis):
    _data = fakes.FakeDumpTask.create_one()

    columns = (
        'created_at',
        'destination_type',
        'exception_strategy',
        'last_transfer_timestamp',
        'obs_destination_description',
        'partitions',
        'status',
        'stream_name',
        'task_id',
        'task_name'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateDumpTask, self).setUp()

        self.cmd = dump_task.CreateDumpTask(self.app, None)
        self.client.create_dump_task = mock.Mock()
        self.client.get_dump_task = mock.Mock(return_value=self._data)

    def test_create(self):
        arglist = [
            'test-stream',
            'test-dump-task',
            '--destination-type', 'OBS',
            '--agency-name', '4',
            '--deliver-time-interval', '5',
            '--consumer-strategy', 'LATEST',
            '--file-prefix', '7',
            '--partition-format', 'yyyy',
            '--obs-bucket-path', '8',
            '--destination-file-type', 'Text',
            '--record-delimiter', '\\n',
        ]
        verifylist = [
            ('streamName', 'test-stream'),
            ('taskName', 'test-dump-task'),
            ('destination_type', 'OBS'),
            ('agency_name', '4'),
            ('deliver_time_interval', 5),
            ('consumer_strategy', 'LATEST'),
            ('file_prefix', '7'),
            ('partition_format', 'yyyy'),
            ('obs_bucket_path', '8'),
            ('destination_file_type', 'Text'),
            ('record_delimiter', '\\n'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        attrs = {}
        attrs.update(
            destination_type='OBS',
            obs_destination_descriptor={
                'task_name': 'test-dump-task',
                'agency_name': '4',
                'deliver_time_interval': 5,
                'consumer_strategy': 'LATEST',
                'file_prefix': '7',
                'partition_format': 'yyyy',
                'obs_bucket_path': '8',
                'destination_file_type': 'Text',
                'record_delimiter': '\\n'
            }
        )

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_dump_task.assert_called_with('test-stream', **attrs)
        self.assertEqual(self.columns, columns)


class TestShowDumpTask(fakes.TestDis):
    _data = fakes.FakeDumpTask.create_one()

    columns = (
        'created_at',
        'destination_type',
        'exception_strategy',
        'last_transfer_timestamp',
        'obs_destination_description',
        'partitions',
        'status',
        'stream_name',
        'task_id',
        'task_name'
    )
    data = fakes.gen_data(_data, columns, dump_task._formatters)

    def setUp(self):
        super(TestShowDumpTask, self).setUp()

        self.cmd = dump_task.ShowDumpTask(self.app, None)

        self.client.get_dump_task = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(tests_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_show(self):
        arglist = [
            self._data.stream_name,
            self._data.task_name,
        ]

        verifylist = [
            ('streamName', self._data.stream_name),
            ('taskName', self._data.task_name),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.get_dump_task.assert_called_with(
            self._data.stream_name, self._data.task_name
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            self._data.stream_name,
            'unexist_dis_dump_task',
        ]

        verifylist = [
            ('streamName', self._data.stream_name),
            ('taskName', 'unexist_dis_dump_task'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        get_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.get_dump_task = (
            mock.Mock(side_effect=get_mock_result)
        )
        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.get_dump_task.assert_called_with(
            self._data.stream_name, 'unexist_dis_dump_task')


class TestDeleteDumpTask(fakes.TestDis):

    _data = fakes.FakeDumpTask.create_multiple(2)

    def setUp(self):
        super(TestDeleteDumpTask, self).setUp()

        self.client.delete_dump_task = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = dump_task.DeleteDumpTask(self.app, None)

    def test_delete(self):
        arglist = [
            self._data[0].stream_name,
            self._data[0].task_name,
        ]

        verifylist = [
            ('streamName', self._data[0].stream_name),
            ('taskName', [self._data[0].task_name]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_dump_task.assert_called_with(
            self._data[0].stream_name, self._data[0].task_name)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = [
            self._data[0].stream_name,
            self._data[0].task_name,
            self._data[1].task_name
        ]

        verifylist = [
            ('streamName', self._data[0].stream_name),
            ('taskName', [self._data[0].task_name, self._data[1].task_name]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        calls = [
            call(self._data[0].stream_name, self._data[0].task_name),
            call(self._data[0].stream_name, self._data[1].task_name)
        ]
        self.client.delete_dump_task.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._data[0].stream_name,
            self._data[0].task_name,
            'unexist_dis_dump_task',
        ]
        verifylist = [
            ('streamName', self._data[0].stream_name),
            ('taskName', [self._data[0].task_name, 'unexist_dis_dump_task']),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        delete_mock_result = [None, exceptions.CommandError]
        self.client.delete_dump_task = (
            mock.Mock(side_effect=delete_mock_result)
        )
        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual(
                '1 of 2 dump task(s) failed to delete.', str(e))

        calls = [
            call(self._data[0].stream_name, self._data[0].task_name),
            call(self._data[0].stream_name, 'unexist_dis_dump_task')
        ]
        self.client.delete_dump_task.assert_has_calls(calls)


class TestStartDumpTask(fakes.TestDis):

    def setUp(self):
        super(TestStartDumpTask, self).setUp()
        self.cmd = dump_task.StartDumpTask(self.app, None)
        self.client.start_dump_task = mock.Mock(return_value=None)

    def test_restart(self):

        arglist = [
            'test-stream',
            'test-dump-taskId',
        ]

        verifylist = [
            ('streamName', 'test-stream'),
            ('taskId', ['test-dump-taskId']),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.start_dump_task.assert_called_with(
            'test-stream',
            ['test-dump-taskId']
        )
        self.assertIsNone(result)


class TestPauseDumpTask(fakes.TestDis):

    def setUp(self):
        super(TestPauseDumpTask, self).setUp()
        self.cmd = dump_task.PauseDumpTask(self.app, None)
        self.client.pause_dump_task = mock.Mock(return_value=None)

    def test_restart(self):

        arglist = [
            'test-stream',
            'test-dump-taskId',
        ]

        verifylist = [
            ('streamName', 'test-stream'),
            ('taskId', ['test-dump-taskId']),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.pause_dump_task.assert_called_with(
            'test-stream',
            ['test-dump-taskId']
        )
        self.assertIsNone(result)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Update Auto-Scaling Policy by using id or an instance of class Policy.
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


policy = "policy_name_or_id"
attrs = {
    "scaling_policy_type": "RECURRENCE",
    "scaling_policy_name": "policy_01",
    "scheduled_policy": {
        "launch_time": "16:00",
        "recurrence_type": "Daily",
        "end_time": "2016-02-08T17:31Z",
        "start_time": "2016-01-08T17:31Z"
    },
    "scaling_policy_action": {
        "operation": "SET",
        "instance_number": 2
    }
}

policy = conn.auto_scaling.find_policy(policy)
conn.auto_scaling.update_policy(policy, **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""DIS Stream v2 action implementations"""
import logging

from osc_lib import utils
# from osc_lib.cli import parseractions
from osc_lib import exceptions
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils
from otcextensions.common import cli_utils


LOG = logging.getLogger(__name__)


_formatters = {
    'obs_destination_descriptor': cli_utils.YamlFormat,
    'obs_destination_description': cli_utils.YamlFormat,
    'partitions': cli_utils.YamlFormat,
    'created_at': cli_utils.UnixTimestampFormatter,
    'last_transfer_timestamp': cli_utils.UnixTimestampFormatter,
}


def _get_columns(item):
    column_map = {}
    hidden = [
        'location',
        'id',
        'stream_id',
    ]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map,
                                                           hidden)


CONSUMER_STRATEGY_CHOICES = ('LATEST', 'TRIM_HORIZON',)


PARTITION_FORMAT_CHOICES = (
    'yyyy',
    'yyyy/MM',
    'yyyy/MM/dd',
    'yyyy/MM/dd/HH',
    'yyyy/MM/dd/HH/mm',
)


RECORD_DELIMITER_CHOICES = (',', ';', '|', '\\n',)


class ListDumpTasks(command.Lister):

    _description = _("List Dump Tasks.")
    display_columns = (
        'Task Name',
        'Task Id',
        'Destination Type',
        'Created At',
        'Status'
    )
    columns = (
        'task_name',
        'task_id',
        'destination_type',
        'created_at',
        'status',
    )

    def get_parser(self, prog_name):
        parser = super(ListDumpTasks, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Specifies the name of the DIS Stream."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        data = client.dump_tasks(parsed_args.streamName)

        return (
            self.display_columns, (
                utils.get_item_properties(
                    s, self.columns, formatters=_formatters
                )
                for s in data
            )
        )


class ShowDumpTask(command.ShowOne):
    _description = _("Query Details of a Dump Task.")

    def get_parser(self, prog_name):
        parser = super(ShowDumpTask, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Specifies the name of the DIS Stream."),
        )
        parser.add_argument(
            'taskName',
            metavar='<taskName>',
            help=_("Specifies the name of Dump Task Name."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        obj = client.get_dump_task(parsed_args.streamName,
                                   parsed_args.taskName)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class CreateDumpTask(command.ShowOne):
    _description = _("Adding a Dump Task.")

    def get_parser(self, prog_name):
        parser = super(CreateDumpTask, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Specifies the name of the DIS Stream."),
        )
        parser.add_argument(
            'taskName',
            metavar='<taskName>',
            help=_("Name of the dump task."),
        )
        parser.add_argument(
            '--destination-type',
            metavar='<destrination_type>',
            default='OBS',
            help=_("Specifies the name of the DIS Stream."),
        )
        parser.add_argument(
            '--agency-name',
            metavar='<agency_name>',
            required=True,
            help=_("Name of the agency created on IAM. DIS uses an "
                   "agency to access your specified resources."),
        )
        parser.add_argument(
            '--deliver-time-interval',
            metavar='<deliver_time_interval>',
            required=True,
            type=int,
            help=_("User-defined interval at which data is imported from "
                   "the current DIS stream into OBS."),
        )
        parser.add_argument(
            '--consumer-strategy',
            dest='consumer_strategy',
            metavar='{' + ','.join(CONSUMER_STRATEGY_CHOICES) + '}',
            type=lambda s: s.upper(),
            choices=CONSUMER_STRATEGY_CHOICES,
            help=_("Offset."
                   "\nLATEST: Maximum offset, indicating that the latest "
                   "data will be extracted."
                   "\nTRIM_HORIZON: Minimum offset, indicating that the "
                   "earliest data will be extracted."
                   "\nDefault value: LATEST."),
        )
        parser.add_argument(
            '--file-prefix',
            metavar='<file_prefix>',
            help=_("Directory to store files that will be dumped to OBS. "
                   "Different directory levels are separated by slashes (/) "
                   "and cannot start with slashes."),
        )
        parser.add_argument(
            '--partition-format',
            dest='partition_format',
            metavar='{' + ','.join(PARTITION_FORMAT_CHOICES) + '}',
            choices=PARTITION_FORMAT_CHOICES,
            help=_("Directory structure of the object file written into OBS. "
                   "The directory structure is in the format of yyyy/MM/dd/"
                   "HH/mm (time at which the dump task was created). "
                   "\nN/A: Leave this parameter empty, indicating that the "
                   "date and time directory is not used."),
        )
        parser.add_argument(
            '--obs-bucket-path',
            metavar='<obs_bucket_path>',
            required=True,
            help=_("Name of the OBS bucket used to store data from the "
                   "DIS stream."),
        )
        parser.add_argument(
            '--destination-file-type',
            metavar='<destination_file_type>',
            help=_("Dump file format. Possible values:"
                   "Text (default)"),
        )
        parser.add_argument(
            '--record-delimiter',
            dest='record_delimiter',
            metavar='{' + ','.join(RECORD_DELIMITER_CHOICES) + '}',
            choices=RECORD_DELIMITER_CHOICES,
            help=_("Delimiter for the dump file, which is used to separate "
                   "the user data that is written into the dump file. "
                   "Default: \\n"),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis

        args_list = (
            'agency_name',
            'deliver_time_interval',
            'consumer_strategy',
            'file_prefix',
            'partition_format',
            'obs_bucket_path',
            'destination_file_type',
            'record_delimiter',
        )
        obs_dest_spec = {
            'task_name': parsed_args.taskName
        }
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                obs_dest_spec[arg] = val

        attrs = {
            'destination_type': parsed_args.destination_type,
            'obs_destination_descriptor': obs_dest_spec
        }

        client.create_dump_task(parsed_args.streamName, **attrs)

        obj = client.get_dump_task(parsed_args.streamName,
                                   parsed_args.taskName)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class DeleteDumpTask(command.Command):

    _description = _("Deletes DIS Dump Task(s).")

    def get_parser(self, prog_name):
        parser = super(DeleteDumpTask, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Name of Dis Stream."),
        )
        parser.add_argument(
            'taskName',
            metavar='<taskName>',
            nargs='+',
            help=_("Name of Dump Task(s) to delete."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        result = 0
        for task_name in parsed_args.taskName:
            try:
                client.delete_dump_task(parsed_args.streamName, task_name)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete dump task with "
                          "name '%(task_name)s': %(e)s"),
                          {'task_name': task_name, 'e': e})
        if result > 0:
            total = len(parsed_args.taskName)
            msg = (_("%(result)s of %(total)s dump task(s) failed "
                   "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)


class StartDumpTask(command.Command):

    _description = _("Start Dump Task(s).")

    def get_parser(self, prog_name):
        parser = super(StartDumpTask, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Name of Dis Stream."),
        )
        parser.add_argument(
            'taskId',
            metavar='<taskId>',
            nargs='+',
            help=_("ID of Dump Task(s) to Start."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        client.start_dump_task(parsed_args.streamName, parsed_args.taskId)


class PauseDumpTask(command.Command):

    _description = _("Pause Dump Task(s).")

    def get_parser(self, prog_name):
        parser = super(PauseDumpTask, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Name of Dis Stream."),
        )
        parser.add_argument(
            'taskId',
            metavar='<taskId>',
            nargs='+',
            help=_("ID of Dump Task(s) to Pause."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        client.pause_dump_task(parsed_args.streamName, parsed_args.taskId)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

from openstack import exceptions

from otcextensions.tests.unit.sdk import base


class TestRdsMixin(base.TestCase):

    flavor = {'id': 1, 'name': 'test'}

    def setUp(self):
        super(TestRdsMixin, self).setUp()

    def test_create_rds_instance(self):
        self.register_uris([
            dict(
                method='GET',
                uri=self.get_rds_url(
                    base_url_append='flavors/MySQL',
                    qs_elements=['version_name=5.7']
                ),
                status_code=200,
                json={"flavors": [
                     {
                         "vcpus": "2",
                         "ram": 8,
                         "id": "1",
                         "spec_code": "rds.mysql.s1.large",
                         "instance_mode": "single",
                         "az_status": {
                             "eu-de-02": "normal",
                             "eu-de-01": "normal",
                             "eu-de-03": "normal"},
                         "version_name": ["5.7"]}]}),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='networks',
                    base_url_append='v2.0',
                    append=['test-network-id']
                ),
                json={'id': 'fake'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='security-groups',
                    base_url_append='v2.0',
                    append=['test-sec_grp-id']
                ),
                json={'id': 'fake'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='routers',
                    base_url_append='v2.0',
                    append=['test-vpc-id']
                ),
                json={'id': 'fake'}
            ),
            dict(
                method='POST',
                uri=self.get_rds_url(
                    resource='instances'
                ),
                status_code=200,
                json={
                    'instance': {'id': 123987},
                    'job_id': '15'
                }
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='jobs',
                    qs_elements=['id=15']
                ),
                status_code=200,
                json={'status': 'completed'}
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='instances',
                    qs_elements=['id=123987']
                ),
                status_code=200,
                json={'instances': [{'id': 123987, 'name': 'inst_name'}]}
            )
        ])

        obj = self.cloud.create_rds_instance(
            availability_zone='test-az-01',
            charge_info={'charge_mode': 'postPaid'},
            configuration='123',
            datastore_type='MySQL',
            datastore_version='5.7',
            disk_encryption_id='234',
            flavor='rds.mysql.s1.large',
            name='inst_name',
            password='testtest',
            port=12345,
            region='test-region',
            router='test-vpc-id',
            security_group='test-sec_grp-id',
            volume_size=100,
            volume_type='ULTRAHIGH',
            network='test-network-id'
        )
        self.assert_calls()

        self.assertEqual('inst_name', obj.name)

    def test_create_rds_instance_ha(self):
        self.register_uris([
            dict(
                method='GET',
                uri=self.get_rds_url(
                    base_url_append='flavors/MySQL',
                    qs_elements=['version_name=5.7']
                ),
                status_code=200,
                json={"flavors": [
                     {
                         "vcpus": "2",
                         "ram": 8,
                         "id": "1",
                         "spec_code": "rds.mysql.s1.large.ha",
                         "instance_mode": "ha",
                         "az_status": {
                             "eu-de-02": "normal",
                             "eu-de-01": "normal",
                             "eu-de-03": "normal"},
                         "version_name": ["5.7"]}]}),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='networks',
                    base_url_append='v2.0',
                    append=['test-network-id']
                ),
                json={'id': 'fake'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='security-groups',
                    base_url_append='v2.0',
                    append=['test-sec_grp-id']
                ),
                json={'id': 'fake'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='routers',
                    base_url_append='v2.0',
                    append=['test-vpc-id']
                ),
                json={'id': 'fake'}
            ),
            dict(
                method='POST',
                uri=self.get_rds_url(
                    resource='instances'
                ),
                status_code=200,
                json={
                    'instance': {'id': 123987},
                    'job_id': '15'
                }
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='jobs',
                    qs_elements=['id=15']
                ),
                status_code=200,
                json={'status': 'completed'}
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='instances',
                    qs_elements=['id=123987']
                ),
                status_code=200,
                json={'instances': [{'id': 123987, 'name': 'inst_name'}]}
            )
        ])

        obj = self.cloud.create_rds_instance(
            availability_zone='test-az-01,az2',
            charge_info={'charge_mode': 'postPaid'},
            configuration='123',
            datastore_type='MySQL',
            datastore_version='5.7',
            disk_encryption_id='234',
            flavor='rds.mysql.s1.large.ha',
            ha_mode='async',
            name='inst_name',
            password='testtest',
            port=12345,
            region='test-region',
            router='test-vpc-id',
            security_group='test-sec_grp-id',
            volume_size=100,
            volume_type='ULTRAHIGH',
            network='test-network-id'
        )

        self.assert_calls()

        self.assertEqual('inst_name', obj.name)

    def test_create_rds_instance_ha_wrong_az(self):
        self.register_uris([
            dict(
                method='GET',
                uri=self.get_rds_url(
                    base_url_append='flavors/MySQL',
                    qs_elements=['version_name=5.7']
                ),
                status_code=200,
                json={"flavors": [
                     {
                         "vcpus": "2",
                         "ram": 8,
                         "id": "1",
                         "spec_code": "rds.mysql.s1.large.ha",
                         "instance_mode": "ha",
                         "az_status": {
                             "eu-de-02": "normal",
                             "eu-de-01": "normal",
                             "eu-de-03": "normal"},
                         "version_name": ["5.7"]}]})
        ])

        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_rds_instance,
            availability_zone='test-az-01',
            charge_info={'charge_mode': 'postPaid'},
            configuration='123',
            datastore_type='MySQL',
            datastore_version='5.7',
            disk_encryption_id='234',
            flavor='rds.mysql.s1.large.ha',
            ha_mode='async',
            name='inst_name',
            password='testtest',
            port=12345,
            region='test-region',
            router='test-vpc-id',
            security_group='test-sec_grp-id',
            volume_size=100,
            volume_type='ULTRAHIGH',
            network='test-network-id'
        )

        self.assert_calls()

    def test_create_rds_instance_ha_wrong_mode_pg(self):
        self.register_uris([
            dict(
                method='GET',
                uri=self.get_rds_url(
                    base_url_append='flavors/postgresql',
                    qs_elements=['version_name=5.7']
                ),
                status_code=200,
                json={"flavors": [
                     {
                         "vcpus": "2",
                         "ram": 8,
                         "id": "1",
                         "spec_code": "rds.pg.s1.large.ha",
                         "instance_mode": "ha",
                         "az_status": {
                             "eu-de-02": "normal",
                             "eu-de-01": "normal",
                             "eu-de-03": "normal"},
                         "version_name": ["5.7"]}]})
        ])

        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_rds_instance,
            availability_zone='test-az-01,az2',
            charge_info={'charge_mode': 'postPaid'},
            configuration='123',
            datastore_type='postgresql',
            datastore_version='5.7',
            disk_encryption_id='234',
            flavor='rds.mysql.s1.large.ha',
            ha_mode='semisync',
            name='inst_name',
            password='testtest',
            port=12345,
            region='test-region',
            router='test-vpc-id',
            security_group='test-sec_grp-id',
            volume_size=100,
            volume_type='ULTRAHIGH',
            network='test-network-id'
        )

        self.assert_calls()

    def test_create_rds_instance_ha_wrong_mode_mysql(self):
        self.register_uris([
            dict(
                method='GET',
                uri=self.get_rds_url(
                    base_url_append='flavors/MySQL',
                    qs_elements=['version_name=5.7']
                ),
                status_code=200,
                json={"flavors": [
                     {
                         "vcpus": "2",
                         "ram": 8,
                         "id": "1",
                         "spec_code": "rds.pg.s1.large.ha",
                         "instance_mode": "ha",
                         "az_status": {
                             "eu-de-02": "normal",
                             "eu-de-01": "normal",
                             "eu-de-03": "normal"},
                         "version_name": ["5.7"]}]})
        ])

        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_rds_instance,
            availability_zone='test-az-01,az2',
            charge_info={'charge_mode': 'postPaid'},
            configuration='123',
            datastore_type='MySQL',
            datastore_version='5.7',
            disk_encryption_id='234',
            flavor='rds.mysql.s1.large.ha',
            ha_mode='sync',
            name='inst_name',
            password='testtest',
            port=12345,
            region='test-region',
            router='test-vpc-id',
            security_group='test-sec_grp-id',
            volume_size=100,
            volume_type='ULTRAHIGH',
            network='test-network-id'
        )

        self.assert_calls()

    def test_create_rds_instance_ha_wrong_mode_mssql(self):
        self.register_uris([
            dict(
                method='GET',
                uri=self.get_rds_url(
                    base_url_append='flavors/sqlserver',
                    qs_elements=['version_name=5.7']
                ),
                status_code=200,
                json={"flavors": [
                     {
                         "vcpus": "2",
                         "ram": 8,
                         "id": "1",
                         "spec_code": "rds.pg.s1.large.ha",
                         "instance_mode": "ha",
                         "az_status": {
                             "eu-de-02": "normal",
                             "eu-de-01": "normal",
                             "eu-de-03": "normal"},
                         "version_name": ["5.7"]}]})
        ])

        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_rds_instance,
            availability_zone='test-az-01,az2',
            charge_info={'charge_mode': 'postPaid'},
            configuration='123',
            datastore_type='sqlserver',
            datastore_version='5.7',
            disk_encryption_id='234',
            flavor='rds.mysql.s1.large.ha',
            ha_mode='async',
            name='inst_name',
            password='testtest',
            port=12345,
            region='test-region',
            router='test-vpc-id',
            security_group='test-sec_grp-id',
            volume_size=100,
            volume_type='ULTRAHIGH',
            network='test-network-id'
        )

        self.assert_calls()

    def test_rds_create_replica(self):
        self.register_uris([
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='instances',
                    qs_elements=['id=fake_name']
                ),
                status_code=200,
                json={'instances': [{
                    'id': 123987,
                    'name': 'fake_name',
                    'datastore': {'type': 'MySQL', 'version': '5.7'}}]}
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    base_url_append='flavors/MySQL',
                    qs_elements=['version_name=5.7']
                ),
                status_code=200,
                json={"flavors": [
                     {
                         "vcpus": "2",
                         "ram": 8,
                         "id": "1",
                         "spec_code": "rds.mysql.s1.large.replica",
                         "instance_mode": "replica",
                         "az_status": {
                             "eu-de-02": "normal",
                             "eu-de-01": "normal",
                             "eu-de-03": "normal"},
                         "version_name": ["5.7"]}]}
            ),
            dict(
                method='POST',
                uri=self.get_rds_url(
                    resource='instances'
                ),
                status_code=200,
                json={
                    'instance': {'id': 1239876},
                    'job_id': '15'
                }
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='jobs',
                    qs_elements=['id=15']
                ),
                status_code=200,
                json={'status': 'completed'}
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='instances',
                    qs_elements=['id=1239876']
                ),
                status_code=200,
                json={'instances': [{
                    'id': 1239876,
                    'name': 'copy_name'}]}
            )
        ])

        obj = self.cloud.create_rds_instance(
            availability_zone='test-az-01',
            configuration='123',
            disk_encryption_id='234',
            flavor='rds.mysql.s1.large.replica',
            name='inst_name',
            replica_of='fake_name',
            volume_size=100,
            volume_type='ULTRAHIGH'
        )

        self.assert_calls()

        self.assertEqual('copy_name', obj.name)

    def test_create_rds_instance_replica_exception(self):
        """setting port with replica not allowed"""

        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_rds_instance,
            configuration='123',
            datastore_type='MySQL',
            datastore_version='5.7',
            disk_encryption_id='234',
            flavor='test-flavor',
            replica_of='fake_name',
            ha_mode='semisync',
            name='inst_name',
            port=5432,
            region='test-region',
            volume_size=100,
            volume_type='ULTRAHIGH',
        )

        self.assert_calls()

    def test_rds_create_from_instance_pir(self):
        self.register_uris([
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='instances',
                    qs_elements=['id=source_instance']
                ),
                status_code=200,
                json={'instances': [{
                    'id': 123987,
                    'name': 'source_instance',
                    'datastore': {'type': 'MySQL', 'version': '5.7'}}]}
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    base_url_append='flavors/MySQL',
                    qs_elements=['version_name=5.7']
                ),
                status_code=200,
                json={"flavors": [
                     {
                         "vcpus": "2",
                         "ram": 8,
                         "id": "1",
                         "spec_code": "rds.mysql.s1.large",
                         "instance_mode": "single",
                         "az_status": {
                             "eu-de-02": "normal",
                             "eu-de-01": "normal",
                             "eu-de-03": "normal"},
                         "version_name": ["5.7"]}]}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='networks',
                    base_url_append='v2.0',
                    append=['test-network-id']
                ),
                json={'id': 'fake'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='security-groups',
                    base_url_append='v2.0',
                    append=['test-sec_grp-id']
                ),
                json={'id': 'fake'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='routers',
                    base_url_append='v2.0',
                    append=['test-vpc-id']
                ),
                json={'id': 'fake'}
            ),

            dict(
                method='POST',
                uri=self.get_rds_url(
                    resource='instances'
                ),
                status_code=200,
                json={
                    'instance': {'id': 1239876},
                    'job_id': '15'
                },
                validate=dict(
                    json={
                        'availability_zone': 'test-az-01',
                        'configuration_id': '123',
                        'datastore': {'type': 'MySQL', 'version': '5.7'},
                        'disk_encryption_id': '234',
                        'flavor_ref': 'rds.mysql.s1.large',
                        'name': 'inst_name',
                        'port': 12345,
                        'region': 'test-region',
                        'restore_point': {'instance_id': 123987,
                                          'restore_time': 'abcde',
                                          'type': 'timestamp'},
                        'security_group_id': 'fake',
                        'subnet_id': 'fake',
                        'volume': {'size': 100, 'type': 'ULTRAHIGH'},
                        'vpc_id': 'fake'}
                )
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='jobs',
                    qs_elements=['id=15']
                ),
                status_code=200,
                json={'status': 'completed'}
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='instances',
                    qs_elements=['id=1239876']
                ),
                status_code=200,
                json={'instances': [{
                    'id': 1239876,
                    'name': 'inst_name'}]}
            )
        ])

        obj = self.cloud.create_rds_instance(
            availability_zone='test-az-01',
            configuration='123',
            datastore_type='MySQL',
            datastore_version='5.7',
            disk_encryption_id='234',
            flavor='rds.mysql.s1.large',
            from_instance='source_instance',
            name='inst_name',
            network='test-network-id',
            port=12345,
            region='test-region',
            restore_time='abcde',
            router='test-vpc-id',
            security_group='test-sec_grp-id',
            volume_size=100,
            volume_type='ULTRAHIGH'
        )

        self.assert_calls()

        self.assertEqual('inst_name', obj.name)

    def test_rds_create_from_backup(self):
        self.register_uris([
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='instances',
                    qs_elements=['id=source_instance']
                ),
                status_code=200,
                json={'instances': [{
                    'id': 123987,
                    'name': 'source_instance',
                    'datastore': {'type': 'MySQL', 'version': '5.7'}}]}
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='backups',
                    qs_elements=['instance_id=123987',
                                 'backup_id=source_backup']
                ),
                status_code=200,
                json={'backups': [{
                    'id': 12,
                    'name': 'source_backup',
                    'datastore': {'type': 'MySQL', 'version': '5.7'}}]}
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    base_url_append='flavors/MySQL',
                    qs_elements=['version_name=5.7']
                ),
                status_code=200,
                json={"flavors": [
                     {
                         "vcpus": "2",
                         "ram": 8,
                         "id": "1",
                         "spec_code": "rds.mysql.s1.large",
                         "instance_mode": "single",
                         "az_status": {
                             "eu-de-02": "normal",
                             "eu-de-01": "normal",
                             "eu-de-03": "normal"},
                         "version_name": ["5.7"]}]}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='networks',
                    base_url_append='v2.0',
                    append=['test-network-id']
                ),
                json={'id': 'fake'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='security-groups',
                    base_url_append='v2.0',
                    append=['test-sec_grp-id']
                ),
                json={'id': 'fake'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='routers',
                    base_url_append='v2.0',
                    append=['test-vpc-id']
                ),
                json={'id': 'fake'}
            ),

            dict(
                method='POST',
                uri=self.get_rds_url(
                    resource='instances'
                ),
                status_code=200,
                json={
                    'instance': {'id': 1239876},
                    'job_id': '15'
                },
                validate=dict(
                    json={
                        'availability_zone': 'test-az-01',
                        'configuration_id': '123',
                        'datastore': {'type': 'MySQL', 'version': '5.7'},
                        'disk_encryption_id': '234',
                        'flavor_ref': 'rds.mysql.s1.large',
                        'name': 'inst_name',
                        'port': 12345,
                        'region': 'test-region',
                        'restore_point': {
                            'backup_id': 12,
                            'instance_id': 123987,
                            'type': 'backup'},
                        'security_group_id': 'fake',
                        'subnet_id': 'fake',
                        'volume': {'size': 100, 'type': 'ULTRAHIGH'},
                        'vpc_id': 'fake'}
                )
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='jobs',
                    qs_elements=['id=15']
                ),
                status_code=200,
                json={'status': 'completed'}
            ),
            dict(
                method='GET',
                uri=self.get_rds_url(
                    resource='instances',
                    qs_elements=['id=1239876']
                ),
                status_code=200,
                json={'instances': [{
                    'id': 1239876,
                    'name': 'inst_name'}]}
            )
        ])

        obj = self.cloud.create_rds_instance(
            availability_zone='test-az-01',
            backup='source_backup',
            configuration='123',
            datastore_type='MySQL',
            datastore_version='5.7',
            disk_encryption_id='234',
            flavor='rds.mysql.s1.large',
            from_instance='source_instance',
            name='inst_name',
            network='test-network-id',
            port=12345,
            region='test-region',
            router='test-vpc-id',
            security_group='test-sec_grp-id',
            volume_size=100,
            volume_type='ULTRAHIGH'
        )

        self.assert_calls()

        self.assertEqual('inst_name', obj.name)

    def test_rds_create_from_backup_no_instance(self):
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_rds_instance,
            availability_zone='test-az-01',
            backup='source_backup',
            configuration='123',
            datastore_type='MySQL',
            datastore_version='5.7',
            disk_encryption_id='234',
            ha_mode='semisync',
            flavor='rds.mysql.s1.large',
            name='inst_name',
            network='test-network-id',
            port=12345,
            region='test-region',
            router='test-vpc-id',
            security_group='test-sec_grp-id',
            volume_size=100,
            volume_type='ULTRAHIGH'
        )

        self.assert_calls()

    def test_rds_create_primary_missing_params(self):
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_rds_instance,
            availability_zone='test-az-01',
            configuration='123',
            datastore_type='MySQL',
            datastore_version='5.7',
            disk_encryption_id='234',
            ha_mode='semisync',
            flavor='rds.mysql.s1.large',
            name='inst_name',
            port=12345,
            region='test-region',
            router='test-vpc-id',
            volume_size=100,
            volume_type='ULTRAHIGH'
        )

        self.assert_calls()

    def test_rds_create_primary_missing_datastore(self):
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_rds_instance,
            availability_zone='test-az-01',
            configuration='123',
            disk_encryption_id='234',
            ha_mode='semisync',
            flavor='rds.mysql.s1.large',
            name='inst_name',
            network='test-network-id',
            port=12345,
            region='test-region',
            router='test-vpc-id',
            security_group='test-sec_grp-id',
            volume_size=100,
            volume_type='ULTRAHIGH'
        )

        self.assert_calls()



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.sdrs.v1 import active_domains
from otcextensions.tests.unit.osclient.sdrs.v1 import fakes


class TestActiveDomains(fakes.TestSDRS):
    object = fakes.FakeActiveDomain.create_multiple(2)

    columns = ('ID', 'name', 'description', 'sold_out',
               'local_replication_cluster', 'remote_replication_cluster')

    flat_data = [(active_domains._flatten_domain(obj)) for obj in object]

    data = [
        (flat_d['id'],
         flat_d['name'],
         flat_d['description'],
         flat_d['sold_out'],
         flat_d['local_replication_cluster'],
         flat_d['remote_replication_cluster']
         ) for flat_d in flat_data]

    def setUp(self):
        super(TestActiveDomains, self).setUp()
        self.cmd = active_domains.ListDomain(self.app, None)
        self.client.get_domains = mock.Mock(self.object)
        self.client.api_mock = self.client.get_domains

    def test_flatten(self):
        obj = fakes.FakeActiveDomain.create_one()
        flat_data = active_domains._flatten_domain(obj)

        data = (
            flat_data['id'],
            flat_data['name'],
            flat_data['description'],
            flat_data['sold_out'],
            flat_data['local_replication_cluster'],
            flat_data['remote_replication_cluster']
        )

        cmp_data = (
            obj['domains'][0].id,
            obj['domains'][0].name,
            obj['domains'][0].description,
            obj['domains'][0].sold_out,
            obj['domains'][0].local_replication_cluster.availability_zone,
            obj['domains'][0].remote_replication_cluster.availability_zone
        )

        self.assertEqual(data, cmp_data)

    def test_default(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.get_domains.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        new_list = list(data)
        self.assertEqual(self.data, new_list)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Datastore(resource.Resource):

    base_path = '/datastores/%(database_name)s'
    resources_key = 'dataStores'

    # capabilities
    allow_list = True

    database_name = resource.URI('database_name')

    # Indicates the database version ID. Its value is unique.
    # :*Type:string*
    id = resource.Body('id')
    # Indicates the database version.
    # :*Type:string*
    name = resource.Body('name')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource


class Message(resource.Resource):

    resource_key = 'message'
    base_path = '/queues/%(queue_id)s/groups/%(group_id)s/messages'

    allow_list = True

    _query_mapping = resource.QueryParameters(
        'max_msgs', 'time_wait', 'ack_wait'
    )

    #: Queue id
    queue_id = resource.URI('queue_id')
    #: Group ID (part of URL for listing)
    group_id = resource.URI('group_id')

    # Properties
    #: Attributes
    attributes = resource.Body('attributes', type=dict)
    #: Message body
    body = resource.Body('body')
    #: Error info
    error = resource.Body('error')
    #: Error code
    error_code = resource.Body('error_code')
    #: Message handler (ID)
    handler = resource.Body('handler', alternate_id=True)
    #: State (0 - success)
    state = resource.Body('state', type=int)

    @classmethod
    def list(cls, session, paginated=True, base_path=None,
             allow_unknown_params=False, **params):

        microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path
        params = cls._query_mapping._validate(
            params, base_path=base_path,
            allow_unknown_params=allow_unknown_params)
        query_params = cls._query_mapping._transpose(params, cls)
        uri = base_path % params

        while uri:
            # Copy query_params due to weird mock unittest interactions
            response = session.get(
                uri,
                headers={"Accept": "application/json"},
                params=query_params.copy(),
                microversion=microversion)
            exceptions.raise_from_response(response)
            data = response.json()
            resources = data

            if not isinstance(resources, list):
                resources = [resources]

            for raw_resource in resources:
                attrs = raw_resource.pop('message')
                handler = raw_resource.pop('handler')
                value = cls.existing(
                    microversion=microversion,
                    connection=session._get_connection(),
                    id=handler,
                    **attrs)
                yield value

            if not resources:
                return


class Messages(resource.Resource):

    resources_key = 'messages'
    base_path = '/queues/%(queue_id)s/messages'

    # capabilities
    allow_create = True
    allow_list = True

    # Properties
    #: Queue id
    queue_id = resource.URI('queue_id')
    #: Messages
    messages = resource.Body('messages', type=list, list_type=Message)
    #: Whether message ID should be returned
    return_id = resource.Body('returnId', type=bool)
